unit PulseOutUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls;

type
  TPulseOutForm = class(TForm)
    PulseCountLabel: TLabel;
    PulseCountTrack: TTrackBar;
    PulseDirGroup: TGroupBox;
    PulseDirCombo: TComboBox;
    TimePages: TPageControl;
    FreqDutySheet: TTabSheet;
    FreqLabel: TLabel;
    DutyLabel: TLabel;
    FreqTrack: TTrackBar;
    DutyTrack: TTrackBar;
    HiLoSheet: TTabSheet;
    ActiveSideLabel: TLabel;
    InactiveSideLabel: TLabel;
    ActiveSideTrack: TTrackBar;
    InactiveSideTrack: TTrackBar;
    GoGroup: TGroupBox;
    GoButtonB0: TButton;
    GoButtonB1: TButton;
    GoButtonB2: TButton;
    GoButtonB3: TButton;
    GoButtonB4: TButton;
    GoButtonB5: TButton;
    GoButtonB6: TButton;
    GoButtonB7: TButton;
    procedure FormCreate(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure PulseCountTrackChange(Sender: TObject);
    procedure FreqTrackChange(Sender: TObject);
    procedure DutyTrackChange(Sender: TObject);
    procedure ActiveSideTrackChange(Sender: TObject);
    procedure InactiveSideTrackChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  protected
    bProgrammaticChangeDuty: Boolean;
    LastUserDuty: Double;

    procedure CreateParams(var Params: TCreateParams); override;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  PulseOutForm: TPulseOutForm;

implementation

{$R *.DFM}

uses
  AIOWDM,
  SampleGlobals;

const
  PulseCtIsPWM = $10000;

procedure TPulseOutForm.CreateParams(var Params: TCreateParams);
begin
  inherited;

  Params.WndParent := Application.MainForm.Handle;
end;

procedure TPulseOutForm.FormCreate(Sender: TObject);
begin
  LoadDefaultFont(Font);

  AutoSize := True;
  HandleNeeded;
  AutoSize := False;

  PulseDirCombo.ItemIndex := 0;

  LastUserDuty := 1/2;

  FreqTrackChange(FreqTrack);
  DutyTrackChange(DutyTrack);
end;

procedure TPulseOutForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  PulseOutForm := nil;
  Action := caFree;
end;

procedure BitGo(iBit, PulseCt: Integer; bHighGoing: Boolean; ActiveSide, InactiveSide: Word);
var
  BitOff: LongWord;
  PCS: Byte;

begin
  BitOff := $100 + $100 * iBit;

  PCS := $00;
  if bHighGoing then PCS := PCS or $04; //Bit 2 high = high-going pulses, bit 2 low = low-going pulses.

  if PulseCt = PulseCtIsPWM then begin
    PCS := PCS or $02; //OR in "forever" bit.
  end
  else begin
    OutBAR16(CardNum, $81, BitOff + $8, PulseCt);
  end;
  OutBAR8(CardNum, $81, BitOff + $10, PCS);

  //The "active" side is the low side for a low-going pulse, or the high side for a high-going pulse.
  OutBAR16(CardNum, $81, BitOff + $24, ActiveSide);
  //The "inactive" side is the high side for a low-going pulse, or the low side for a high-going pulse.
  OutBAR16(CardNum, $81, BitOff + $20, InactiveSide);


  begin
    //If you know you don't want to change any of the parameters above, then you can
    //just issue this one "go". You can alternately change some of the parameters
    //above, then issue this "go".
    //For example, leave pulse count and PCS set to PWM, and just change the active
    //side and inactive side to adjust the PWM duty cycle.
    OutBAR32(CardNum, $81, $50, 1 shl iBit);
  end;
end;

procedure TPulseOutForm.GoButtonClick(Sender: TObject);
var
  iBit: Integer;
  ActiveSide, InactiveSide: Word;
  FullPeriod: Integer;
  IOGroupMask, Config: Byte;

begin
  iBit := (Sender as TButton).Tag;

  case iBit of
    0..7: IOGroupMask := CONFIGDIR_PORTA;
    8..15: IOGroupMask := CONFIGDIR_PORTA;
    16..19: IOGroupMask := CONFIGDIR_PORTCLO;
    20..23: IOGroupMask := CONFIGDIR_PORTCHI;
    else Exit;
  end;

  //If it's not already, then set the bit's group as output. Leave the rest as they were.
  Config := RelInPortB(CardNum, 3);
  if (Config and IOGroupMask) <> 0 then begin
    Config := Config and not IOGroupMask;
    RelOutPortB(CardNum, 3, Config);
  end;

  if TimePages.ActivePage = FreqDutySheet then begin
    FullPeriod := $10000 - FreqTrack.Position;
    ActiveSide := DutyTrack.Position;
    InactiveSide := FullPeriod - ActiveSide;
  end
  else begin
    ActiveSide := ActiveSideTrack.Position;
    InactiveSide := InactiveSideTrack.Position;
  end;

  BitGo(
    iBit,
    PulseCountTrack.Position,
    PulseDirCombo.ItemIndex <> 0,
    ActiveSide,
    InactiveSide
  );
end;

procedure TPulseOutForm.PulseCountTrackChange(Sender: TObject);
var
  PulseCt: Integer;

begin
  PulseCt := PulseCountTrack.Position;
  if PulseCt = PulseCtIsPWM then begin
    PulseCountLabel.Caption := 'PWM';
  end
  else begin
    if PulseCt = 1 then
      PulseCountLabel.Caption := '1 Pulse'
    else
      PulseCountLabel.Caption := IntToStr(PulseCt) + ' Pulses'
    ;
  end;
end;

procedure TPulseOutForm.FreqTrackChange(Sender: TObject);
var
  FullPeriod, TenthPeriod: Integer;

begin
  FullPeriod := $10000 - FreqTrack.Position;
  FreqLabel.Caption := '&Frequency: ' + FreqToStr(BaseClockHz / FullPeriod);

  bProgrammaticChangeDuty := True;
  try
    DutyTrack.Max := FullPeriod - 1;
    DutyTrack.Position := Round(LastUserDuty * FullPeriod);
    TenthPeriod := Round(FullPeriod / 10);
    DutyTrack.Frequency := TenthPeriod;
    DutyTrack.PageSize := TenthPeriod;
  finally
    bProgrammaticChangeDuty := False;
  end;
end;

procedure TPulseOutForm.DutyTrackChange(Sender: TObject);
var
  FullPeriod: Integer;
  Duty: Double;

begin
  FullPeriod := DutyTrack.Max - DutyTrack.Min + 2;
  Duty := DutyTrack.Position / FullPeriod;

  if not bProgrammaticChangeDuty then LastUserDuty := Duty;

  DutyLabel.Caption := '&Duty Cycle: ' + FloatToStr(Round(10000 * Duty) / 100) + '%';
end;

procedure TPulseOutForm.ActiveSideTrackChange(Sender: TObject);
begin
  ActiveSideLabel.Caption := '&Active Side: ' + TimeIntervalToStr(ActiveSideTrack.Position / BaseClockHz);
end;

procedure TPulseOutForm.InactiveSideTrackChange(Sender: TObject);
begin
  InactiveSideLabel.Caption := '&Inactive Side: ' + TimeIntervalToStr(InactiveSideTrack.Position / BaseClockHz);
end;

end.
