unit SwitchBoxUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TSwitchBoxForm = class(TForm)
    PortGroupA: TGroupBox;
    ReadTime: TTimer;
    InputRadioA: TRadioButton;
    OutputRadioA: TRadioButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    BitLabelA0: TLabel;
    HighButtonA0: TButton;
    StateLabelA0: TLabel;
    LowButtonA0: TButton;
    GroupBox1: TGroupBox;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    BitLabelB0: TLabel;
    StateLabelB0: TLabel;
    InputRadioB: TRadioButton;
    OutputRadioB: TRadioButton;
    HighButtonB0: TButton;
    LowButtonB0: TButton;
    GroupBox2: TGroupBox;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    GroupBox3: TGroupBox;
    OutputRadioCLo: TRadioButton;
    InputRadioCLo: TRadioButton;
    GroupBox4: TGroupBox;
    OutputRadioCHi: TRadioButton;
    InputRadioCHi: TRadioButton;
    BitLabelC0: TLabel;
    StateLabelC0: TLabel;
    HighButtonC0: TButton;
    LowButtonC0: TButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure DirRadioClick(Sender: TObject);
    procedure ReadTimeTimer(Sender: TObject);
    procedure HighButtonClick(Sender: TObject);
    procedure LowButtonClick(Sender: TObject);
  protected
    IsProgrammaticChange: Integer;
    DIO: packed record
    case Integer of
      0: (Data: LongWord);
      1: (ByByte: array[0..3] of Byte);
      2: (PortA, PortB, PortC, Config: Byte);
    end;
    BitControl: array[0..2, 0..7] of record
      HighButton, LowButton: TButton;
      StateLabel: TLabel;
    end;

    procedure CreateParams(var Params: TCreateParams); override;
    function bBitIsInput(BitIndex: Integer): Boolean;
    procedure SetPortDirsFromConfig;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  SwitchBoxForm: TSwitchBoxForm;

implementation

{$R *.DFM}

uses
  AIOWDM,
  SampleGlobals;

procedure TSwitchBoxForm.CreateParams(var Params: TCreateParams);
begin
  inherited;

  Params.WndParent := Application.MainForm.Handle;
end;

procedure TSwitchBoxForm.FormCreate(Sender: TObject);

  procedure CopyBitControls(iPort: Integer; BaseBitLabel: TLabel);
  var
    iBit: Integer;
    BitLabel: TLabel;

  begin
    for iBit := 1 to 7 do begin
      BitLabel := TLabel(CopyControlToIndex(Self, BaseBitLabel, iBit, tdLeft));
      BitLabel.Caption := IntToStr(iBit);
      BitControl[iPort, iBit].HighButton := TButton(CopyControlToIndex(Self, BitControl[iPort, 0].HighButton, iBit, tdLeft));
      BitControl[iPort, iBit].LowButton  := TButton(CopyControlToIndex(Self, BitControl[iPort, 0].LowButton , iBit, tdLeft));
      BitControl[iPort, iBit].StateLabel :=  TLabel(CopyControlToIndex(Self, BitControl[iPort, 0].StateLabel, iBit, tdLeft));
    end;
  end;

begin
  LoadDefaultFont(Font);

  Inc(IsProgrammaticChange);
  try
    BitControl[0, 0].HighButton := HighButtonA0;
    BitControl[0, 0].LowButton  :=  LowButtonA0;
    BitControl[0, 0].StateLabel := StateLabelA0;
    CopyBitControls(0, BitLabelA0);
    BitControl[1, 0].HighButton := HighButtonB0;
    BitControl[1, 0].LowButton  :=  LowButtonB0;
    BitControl[1, 0].StateLabel := StateLabelB0;
    CopyBitControls(1, BitLabelB0);
    BitControl[2, 0].HighButton := HighButtonC0;
    BitControl[2, 0].LowButton  :=  LowButtonC0;
    BitControl[2, 0].StateLabel := StateLabelC0;
    CopyBitControls(2, BitLabelC0);
  finally
    Dec(IsProgrammaticChange);
  end;

  DIO.Data := RelInPortL(CardNum, 0);
  SetPortDirsFromConfig;

  ReadTime.Enabled := True;
end;

procedure TSwitchBoxForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SwitchBoxForm := nil;
  Action := caFree;
end;

procedure TSwitchBoxForm.DirRadioClick(Sender: TObject);
var
  iBit: Integer;
  oConfig: Byte;

begin
  if IsProgrammaticChange = 0 then begin
    oConfig := DIO.Config;
    DIO.Config := $80; //Clear the dir bits.
    if InputRadioA.Checked   then DIO.Config := DIO.Config or CONFIGDIR_PORTA  ;
    if InputRadioB.Checked   then DIO.Config := DIO.Config or CONFIGDIR_PORTB  ;
    if InputRadioCLo.Checked then DIO.Config := DIO.Config or CONFIGDIR_PORTCLO;
    if InputRadioCHi.Checked then DIO.Config := DIO.Config or CONFIGDIR_PORTCHI;

    if DIO.Config <> oConfig then RelOutPortB(CardNum, 3, DIO.Config);
  end;

  for iBit := 0 to 7 do begin
    BitControl[0, iBit].HighButton.Visible := (DIO.Config and CONFIGDIR_PORTA) = 0;
    BitControl[0, iBit].LowButton.Visible  := (DIO.Config and CONFIGDIR_PORTA) = 0;
    BitControl[1, iBit].HighButton.Visible := (DIO.Config and CONFIGDIR_PORTB) = 0;
    BitControl[1, iBit].LowButton.Visible  := (DIO.Config and CONFIGDIR_PORTB) = 0;
  end;
  for iBit := 0 to 3 do begin
    BitControl[2, iBit].HighButton.Visible := (DIO.Config and CONFIGDIR_PORTCLO) = 0;
    BitControl[2, iBit].LowButton.Visible  := (DIO.Config and CONFIGDIR_PORTCLO) = 0;
  end;
  for iBit := 4 to 7 do begin
    BitControl[2, iBit].HighButton.Visible := (DIO.Config and CONFIGDIR_PORTCHI) = 0;
    BitControl[2, iBit].LowButton.Visible  := (DIO.Config and CONFIGDIR_PORTCHI) = 0;
  end;
end;

procedure TSwitchBoxForm.ReadTimeTimer(Sender: TObject);
var
  iPort, iBit: Integer;
  BitStr: String;
  oConfig: Byte;

begin
  oConfig := DIO.Config;
  DIO.Data := RelInPortL(CardNum, 0);

  if DIO.Config <> oConfig then begin
    //Somebody else has changed port direction(s), update form to match.
    SetPortDirsFromConfig;
  end;

  for iPort := 0 to 2 do
    for iBit := 0 to 7 do begin
      if (DIO.ByByte[iPort] and (1 shl iBit)) = 0 then
        BitStr := '0'
      else
        BitStr := '1'
      ;
      BitControl[iPort, iBit].StateLabel.Caption := BitStr;
    end
  ;
end;

procedure TSwitchBoxForm.HighButtonClick(Sender: TObject);
var
  iPort, iBit: Integer;

begin
  iBit := (Sender as TButton).Tag;
  if bBitIsInput(iBit) then Exit;

  iPort := iBit div 8;
  iBit := iBit mod 8;

  DIO.ByByte[iPort] := DIO.ByByte[iPort] or (1 shl iBit);
  RelOutPortB(CardNum, iPort, DIO.ByByte[iPort]);
end;

procedure TSwitchBoxForm.LowButtonClick(Sender: TObject);
var
  iPort, iBit: Integer;

begin
  iBit := (Sender as TButton).Tag;
  if bBitIsInput(iBit) then Exit;

  iPort := iBit div 8;
  iBit := iBit mod 8;

  DIO.ByByte[iPort] := DIO.ByByte[iPort] and not (1 shl iBit);
  RelOutPortB(CardNum, iPort, DIO.ByByte[iPort]);
end;

function TSwitchBoxForm.bBitIsInput(BitIndex: Integer): Boolean;
begin
  case BitIndex of
    0..7: Result := InputRadioA.Checked;
    8..15: Result := InputRadioB.Checked;
    16..19: Result := InputRadioCLo.Checked;
    else Result := InputRadioCHi.Checked;
  end;
end;

procedure TSwitchBoxForm.SetPortDirsFromConfig;
begin
  Inc(IsProgrammaticChange);
  try
    if (DIO.Config and CONFIGDIR_PORTA) = 0 then
      OutputRadioA.Checked := True
    else
      InputRadioA.Checked := True
    ;
    if (DIO.Config and CONFIGDIR_PORTB) = 0 then
      OutputRadioB.Checked := True
    else
      InputRadioB.Checked := True
    ;
    if (DIO.Config and CONFIGDIR_PORTCLO) = 0 then
      OutputRadioCLo.Checked := True
    else
      InputRadioCLo.Checked := True
    ;
    if (DIO.Config and CONFIGDIR_PORTCHI) = 0 then
      OutputRadioCHi.Checked := True
    else
      InputRadioCHi.Checked := True
    ;
  finally
    Dec(IsProgrammaticChange);
  end;
end;

end.
